@@ -520,7 +520,12 @@ filesystem_test_() ->
                 application:set_env(lager, handlers, [{lager_test_backend, info}]),
                 application:set_env(lager, error_logger_redirect, false),
                 application:set_env(lager, async_threshold, undefined),
-                lager:start()
+                lager:start(),
+                %% race condition where lager logs its own start up
+                %% makes several tests fail. See test/lager_test_backend
+                %% around line 800 for more information.
+                timer:sleep(5),
+                lager_test_backend:flush()
         end,
         fun(_) ->
                 file:delete("test.log"),
@@ -799,6 +804,64 @@ filesystem_test_() ->
         ]
     }.
 
+trace_files_test_() ->
+    {foreach,
+        fun() ->
+                file:delete("events.log"),
+                file:delete("test.log"),
+                file:delete("debug.log"),
+                error_logger:tty(false),
+                application:load(lager),
+                application:set_env(lager, handlers, [{lager_file_backend, [{file, "test.log"},
+                                                                            {level, error},
+                                                                            {formatter, lager_default_formatter},
+                                                                            {formatter_config, [message, "\n"]}]}]),
+                application:set_env(lager, traces, [
+                                                    { % get default level of debug
+                                                     {lager_file_backend, "debug.log"}, [{module, ?MODULE}]
+                                                    },
+                                                    { % Handler                          Filters              Level
+                                                     {lager_file_backend, "events.log"}, [{module, ?MODULE}], notice
+                                                    }
+                                                   ]
+                                   ),
+                application:set_env(lager, async_threshold, undefined),
+                lager:start()
+        end,
+        fun(_) ->
+                catch ets:delete(lager_config),
+                application:unset_env(lager, traces),
+                application:stop(lager),
+                  
+                file:delete("events.log"),
+                file:delete("test.log"),
+                file:delete("debug.log"),
+                error_logger:tty(true)
+        end,
+        [
+            {"a trace using file backend set up in configuration should work",
+                fun() ->
+                        lager:error("trace test error message"),
+                        lager:info("info message"),
+                        %% not eligible for trace
+                        lager:log(error, self(), "Not trace test message"),
+                        {ok, BinInfo} = file:read_file("events.log"),
+                        ?assertMatch([_, _, "[error]", _, "trace test error message\n"],
+                                     re:split(BinInfo, " ", [{return, list}, {parts, 5}])),
+                        ?assert(filelib:is_regular("test.log")),
+                        {ok, BinInfo2} = file:read_file("test.log"),
+                        ?assertMatch(["trace test error message", "Not trace test message\n"],
+                                     re:split(BinInfo2, "\n", [{return, list}, {parts, 2}])),
+                        ?assert(filelib:is_regular("debug.log")),
+                        %% XXX Aughhhh, wish I could force this to flush somehow...
+                        timer:sleep(1000),
+                        {ok, BinInfo3} = file:read_file("debug.log"),
+                        ?assertEqual(2, length(re:split(BinInfo3, "\n", [{return, list}, trim])))
+                end
+            }
+        ]
+    }.
+
 formatting_test_() ->
     {foreach,
         fun() ->
@@ -808,7 +871,9 @@ formatting_test_() ->
                 application:load(lager),
                 application:set_env(lager, handlers, [{lager_test_backend, info}]),
                 application:set_env(lager, error_logger_redirect, false),
-                lager:start()
+                lager:start(),
+                %% same race condition issue
+                timer:sleep(5)
         end,
         fun(_) ->
                 file:delete("test.log"),
